/*------------------------------------------------*/
/* UART functions                                 */

#include <avr/io.h>
#include <avr/interrupt.h>
#include "integer.h"
#include "uart.h"

#define	BAUD		38400


typedef struct {
	BYTE	wi;
	BYTE	ri;
	BYTE	ct;
	BYTE buff[64];
} FIFO;
static volatile
FIFO TxFifo;



/* Initialize UART */

void uart_init (void)
{
	TxFifo.ri = 0;
	TxFifo.wi = 0;
	TxFifo.ct = 0;

	UBRR0L = F_CPU/BAUD/16-1;
	UCSR0B = _BV(RXEN0)|_BV(TXEN0);
}


/* Get a received character */

BOOL uart_test (void)
{
	return (UCSR0A & _BV(RXC0)) ? TRUE : FALSE;
}


BYTE uart_get (void)
{

	while (bit_is_clear(UCSR0A, RXC0)) ;

	return UDR0;
}


/* Put a character to transmit */

void uart_put (BYTE d)
{
	BYTE i;


	i = TxFifo.wi;
	while (TxFifo.ct >= sizeof(TxFifo.buff)) ;
	TxFifo.buff[i++] = d;
	cli();
	TxFifo.ct++;
	UCSR0B = _BV(RXEN0)|_BV(TXEN0)|_BV(UDRIE0);
	sei();
	TxFifo.wi = i % sizeof(TxFifo.buff);
}


/* UART UDRE interrupt */

ISR(USART_UDRE_vect)
{
	BYTE n, i;


	n = TxFifo.ct;
	if (n) {
		TxFifo.ct = --n;
		i = TxFifo.ri;
		UDR0 = TxFifo.buff[i++];
		TxFifo.ri = i % sizeof(TxFifo.buff);
	}
	if (n == 0)
		UCSR0B = _BV(RXEN0)|_BV(TXEN0);
}

